@@ -17,26 +17,23 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-type lexer
+module Lexer : sig
+  type t
+
+  val of_file : string -> t
+
+  val of_string : ?pos:Lexing.position -> ?filename:string -> string -> t
+
+  val of_channel : in_channel -> t
+end
 
 exception Parsing_error of Parse_info.t
 
-val strip_comment : lexer -> lexer
+val parse : Lexer.t -> Javascript.program
 
-val lexer_from_file : ?rm_comment:bool -> string -> lexer
+val parse' :
+     Lexer.t
+  -> ((Js_token.Annot.t * Parse_info.t) list * Javascript.program) list
+     * (Js_token.t * Lexing.position * Lexing.position) list
 
-val lexer_from_string : ?rm_comment:bool -> ?offset:Parse_info.t -> string -> lexer
-
-val lexer_from_channel : ?rm_comment:bool -> in_channel -> lexer
-
-val lexer_map : (Js_token.token -> Js_token.token) -> lexer -> lexer
-
-val lexer_fold : ('a -> Js_token.token -> 'a) -> 'a -> lexer -> 'a
-
-val lexer_filter : (Js_token.token -> bool) -> lexer -> lexer
-
-val lexer_from_list : Js_token.token list -> lexer
-
-val parse : lexer -> Javascript.program
-
-val parse_expr : lexer -> Javascript.expression
+val parse_expr : Lexer.t -> Javascript.expression
