@@ -31,6 +31,15 @@ external decode_html_entities : js_string t -> js_string t = "caml_js_html_entit
 
 class type cssStyleDeclaration =
   object
+    method setProperty :
+      js_string t -> js_string t -> js_string t optdef -> js_string t meth
+
+    method getPropertyValue : js_string t -> js_string t meth
+
+    method getPropertyPriority : js_string t -> js_string t meth
+
+    method removeProperty : js_string t -> js_string t meth
+
     method animation : js_string t prop
 
     method animationDelay : js_string t prop
@@ -262,6 +271,11 @@ type mouse_button =
   | Middle_button
   | Right_button
 
+type delta_mode =
+  | Delta_pixel
+  | Delta_line
+  | Delta_page
+
 class type event =
   object
     inherit [element] Dom.event
@@ -293,6 +307,10 @@ and mouseEvent =
 
     method screenY : int readonly_prop
 
+    method offsetX : int readonly_prop
+
+    method offsetY : int readonly_prop
+
     method ctrlKey : bool t readonly_prop
 
     method shiftKey : bool t readonly_prop
@@ -338,12 +356,14 @@ and keyboardEvent =
 
     method keyCode : int readonly_prop
 
+    method getModifierState : js_string t -> bool t meth
+
     method keyIdentifier : js_string t optdef readonly_prop
   end
 
 and mousewheelEvent =
   object
-    (* All browsers but Firefox *)
+    (* All modern browsers *)
     inherit mouseEvent
 
     method wheelDelta : int readonly_prop
@@ -351,6 +371,14 @@ and mousewheelEvent =
     method wheelDeltaX : int optdef readonly_prop
 
     method wheelDeltaY : int optdef readonly_prop
+
+    method deltaX : float readonly_prop
+
+    method deltaY : float readonly_prop
+
+    method deltaZ : float readonly_prop
+
+    method deltaMode : delta_mode readonly_prop
   end
 
 and mouseScrollEvent =
@@ -414,6 +442,13 @@ and touch =
     method pageY : int readonly_prop
   end
 
+and submitEvent =
+  object
+    inherit event
+
+    method submitter : element t optdef readonly_prop
+  end
+
 and dragEvent =
   object
     inherit mouseEvent
@@ -421,6 +456,13 @@ and dragEvent =
     method dataTransfer : dataTransfer t readonly_prop
   end
 
+and clipboardEvent =
+  object
+    inherit event
+
+    method clipboardData : dataTransfer t readonly_prop
+  end
+
 and dataTransfer =
   object
     method dropEffect : js_string t prop
@@ -468,6 +510,8 @@ and eventTarget =
 
     method onscroll : ('self t, event t) event_listener writeonly_prop
 
+    method onwheel : ('self t, mousewheelEvent t) event_listener writeonly_prop
+
     method ondragstart : ('self t, dragEvent t) event_listener writeonly_prop
 
     method ondragend : ('self t, dragEvent t) event_listener writeonly_prop
@@ -491,6 +535,14 @@ and eventTarget =
 
     method onanimationcancel : ('self t, animationEvent t) event_listener writeonly_prop
 
+    method ontransitionrun : ('self t, transitionEvent t) event_listener writeonly_prop
+
+    method ontransitionstart : ('self t, transitionEvent t) event_listener writeonly_prop
+
+    method ontransitionend : ('self t, transitionEvent t) event_listener writeonly_prop
+
+    method ontransitioncancel : ('self t, transitionEvent t) event_listener writeonly_prop
+
     method ongotpointercapture : ('self t, pointerEvent t) event_listener writeonly_prop
 
     method onlostpointercapture : ('self t, pointerEvent t) event_listener writeonly_prop
@@ -596,11 +648,31 @@ and animationEvent =
     method pseudoElement : js_string t readonly_prop
   end
 
+and transitionEvent =
+  object
+    inherit event
+
+    method propertyName : js_string t readonly_prop
+
+    method elapsedTime : float readonly_prop
+
+    method pseudoElement : js_string t readonly_prop
+  end
+
 and mediaEvent =
   object
     inherit event
   end
 
+and messageEvent =
+  object
+    inherit event
+
+    method data : Unsafe.any opt readonly_prop
+
+    method source : Unsafe.any opt readonly_prop
+  end
+
 and nodeSelector =
   object
     method querySelector : js_string t -> element t opt meth
@@ -653,6 +725,8 @@ and element =
 
     method textContent : js_string t opt prop
 
+    method innerText : js_string t prop
+
     method clientLeft : int readonly_prop
 
     method clientTop : int readonly_prop
@@ -729,6 +803,12 @@ module Event = struct
 
   let click = Dom.Event.make "click"
 
+  let copy = Dom.Event.make "copy"
+
+  let cut = Dom.Event.make "cut"
+
+  let paste = Dom.Event.make "paste"
+
   let dblclick = Dom.Event.make "dblclick"
 
   let mousedown = Dom.Event.make "mousedown"
@@ -749,6 +829,8 @@ module Event = struct
 
   let mousewheel = Dom.Event.make "mousewheel"
 
+  let wheel = Dom.Event.make "wheel"
+
   let _DOMMouseScroll = Dom.Event.make "DOMMouseScroll"
 
   let touchstart = Dom.Event.make "touchstart"
@@ -835,6 +917,14 @@ module Event = struct
 
   let animationcancel = Dom.Event.make "animationcancel"
 
+  let transitionrun = Dom.Event.make "transitionrun"
+
+  let transitionstart = Dom.Event.make "transitionstart"
+
+  let transitionend = Dom.Event.make "transitionend"
+
+  let transitioncancel = Dom.Event.make "transitioncancel"
+
   let canplay = Dom.Event.make "canplay"
 
   let canplaythrough = Dom.Event.make "canplaythrough"
@@ -855,6 +945,8 @@ module Event = struct
 
   let lostpointercapture = Dom.Event.make "lostpointercapture"
 
+  let message = Dom.Event.make "message"
+
   let pause = Dom.Event.make "pause"
 
   let play = Dom.Event.make "play"
@@ -1011,7 +1103,7 @@ class type formElement =
 
     method reset : unit meth
 
-    method onsubmit : ('self t, event t) event_listener writeonly_prop
+    method onsubmit : ('self t, submitEvent t) event_listener writeonly_prop
   end
 
 class type optGroupElement =
@@ -2064,6 +2156,12 @@ class type document =
 
     method activeElement : element t opt readonly_prop
 
+    method hidden : bool t readonly_prop
+
+    method onfullscreenchange : (document t, event t) event_listener writeonly_prop
+
+    method onwebkitfullscreenchange : (document t, event t) event_listener writeonly_prop
+
     inherit eventTarget
   end
 
@@ -2147,11 +2245,15 @@ class type navigator =
 
     method platform : js_string t readonly_prop
 
+    method vendor : js_string t readonly_prop
+
     method userAgent : js_string t readonly_prop
 
     method language : js_string t optdef readonly_prop
 
     method userLanguage : js_string t optdef readonly_prop
+
+    method maxTouchPoints : int readonly_prop
   end
 
 class type screen =
@@ -2270,13 +2372,13 @@ class type window =
 
     method screen : screen t readonly_prop
 
-    method innerWidth : int optdef readonly_prop
+    method innerWidth : int readonly_prop
 
-    method innerHeight : int optdef readonly_prop
+    method innerHeight : int readonly_prop
 
-    method outerWidth : int optdef readonly_prop
+    method outerWidth : int readonly_prop
 
-    method outerHeight : int optdef readonly_prop
+    method outerHeight : int readonly_prop
 
     method getComputedStyle : #element t -> cssStyleDeclaration t meth
 
@@ -2779,6 +2881,8 @@ module CoerceTo = struct
   let mouseScrollEvent ev = unsafeCoerceEvent Js.Unsafe.global##._MouseScrollEvent ev
 
   let popStateEvent ev = unsafeCoerceEvent Js.Unsafe.global##._PopStateEvent ev
+
+  let messageEvent ev = unsafeCoerceEvent Js.Unsafe.global##._MessageEvent ev
 end
 
 (****)
@@ -2827,36 +2931,17 @@ let buttonPressed (ev : #mouseEvent Js.t) =
       | _ -> No_button)
     (fun x -> x)
 
-let hasMousewheelEvents () =
-  let d = createDiv document in
-  d##setAttribute (Js.string "onmousewheel") (Js.string "return;");
-  Js.typeof (Js.Unsafe.get d (Js.string "onmousewheel")) == Js.string "function"
-
 let addMousewheelEventListenerWithOptions e ?capture ?once ?passive h =
-  if hasMousewheelEvents ()
-  then
-    addEventListenerWithOptions
-      ?capture
-      ?once
-      ?passive
-      e
-      Event.mousewheel
-      (handler (fun (e : mousewheelEvent t) ->
-           let dx = -Optdef.get e##.wheelDeltaX (fun () -> 0) / 40 in
-           let dy = -Optdef.get e##.wheelDeltaY (fun () -> e##.wheelDelta) / 40 in
-           h (e :> mouseEvent t) ~dx ~dy))
-  else
-    addEventListenerWithOptions
-      ?capture
-      ?once
-      ?passive
-      e
-      Event._DOMMouseScroll
-      (handler (fun (e : mouseScrollEvent t) ->
-           let d = e##.detail in
-           if e##.axis == e##._HORIZONTAL_AXIS
-           then h (e :> mouseEvent t) ~dx:d ~dy:0
-           else h (e :> mouseEvent t) ~dx:0 ~dy:d))
+  addEventListenerWithOptions
+    ?capture
+    ?once
+    ?passive
+    e
+    Event.wheel
+    (handler (fun (e : mousewheelEvent t) ->
+         let dx = -Optdef.get e##.wheelDeltaX (fun () -> 0) / 40 in
+         let dy = -Optdef.get e##.wheelDeltaY (fun () -> e##.wheelDelta) / 40 in
+         h (e :> mouseEvent t) ~dx ~dy))
 
 let addMousewheelEventListener e h capt =
   addMousewheelEventListenerWithOptions ~capture:capt e h
@@ -3514,6 +3599,7 @@ let opt_tagged e = Opt.case e (fun () -> None) (fun e -> Some (tagged e))
 type taggedEvent =
   | MouseEvent of mouseEvent t
   | KeyboardEvent of keyboardEvent t
+  | MessageEvent of messageEvent t
   | MousewheelEvent of mousewheelEvent t
   | MouseScrollEvent of mouseScrollEvent t
   | PopStateEvent of popStateEvent t
@@ -3534,7 +3620,11 @@ let taggedEvent (ev : #event Js.t) =
                 (fun () ->
                   Js.Opt.case
                     (CoerceTo.popStateEvent ev)
-                    (fun () -> OtherEvent (ev :> event t))
+                    (fun () ->
+                      Js.Opt.case
+                        (CoerceTo.messageEvent ev)
+                        (fun () -> OtherEvent (ev :> event t))
+                        (fun ev -> MessageEvent ev))
                     (fun ev -> PopStateEvent ev))
                 (fun ev -> MouseScrollEvent ev))
             (fun ev -> MousewheelEvent ev))
