@@ -19,296 +19,322 @@
  *)
 open! Import
 
-type +'a t
+(* This local module [Js] is needed so that the ppx_js extension work within that file. *)
+module Js = struct
+  type +'a t
 
-type (-'a, +'b) meth_callback
+  type (-'a, +'b) meth_callback
 
-module Unsafe = struct
-  type top
+  module Unsafe = struct
+    type top
 
-  type any = top t
+    type any = top t
 
-  type any_js_array = any
+    type any_js_array = any
 
-  external inject : 'a -> any = "%identity"
+    external inject : 'a -> any = "%identity"
 
-  external coerce : _ t -> _ t = "%identity"
+    external coerce : _ t -> _ t = "%identity"
 
-  external get : 'a -> 'b -> 'c = "caml_js_get"
+    external get : 'a -> 'b -> 'c = "caml_js_get"
 
-  external set : 'a -> 'b -> 'c -> unit = "caml_js_set"
+    external set : 'a -> 'b -> 'c -> unit = "caml_js_set"
 
-  external delete : 'a -> 'b -> unit = "caml_js_delete"
+    external delete : 'a -> 'b -> unit = "caml_js_delete"
 
-  external call : 'a -> 'b -> any array -> 'c = "caml_js_call"
+    external call : 'a -> 'b -> any array -> 'c = "caml_js_call"
 
-  external fun_call : 'a -> any array -> 'b = "caml_js_fun_call"
+    external fun_call : 'a -> any array -> 'b = "caml_js_fun_call"
 
-  external meth_call : 'a -> string -> any array -> 'b = "caml_js_meth_call"
+    external meth_call : 'a -> string -> any array -> 'b = "caml_js_meth_call"
 
-  external new_obj : 'a -> any array -> 'b = "caml_js_new"
+    external new_obj : 'a -> any array -> 'b = "caml_js_new"
 
-  external new_obj_arr : 'a -> any_js_array -> 'b = "caml_ojs_new_arr"
+    external new_obj_arr : 'a -> any_js_array -> 'b = "caml_ojs_new_arr"
 
-  external obj : (string * any) array -> 'a = "caml_js_object"
+    external obj : (string * any) array -> 'a = "caml_js_object"
 
-  external equals : 'a -> 'b -> bool = "caml_js_equals"
+    external equals : 'a -> 'b -> bool = "caml_js_equals"
 
-  external pure_expr : (unit -> 'a) -> 'a = "caml_js_pure_expr"
+    external pure_expr : (unit -> 'a) -> 'a = "caml_js_pure_expr"
 
-  external eval_string : string -> 'a = "caml_js_eval_string"
+    external eval_string : string -> 'a = "caml_js_eval_string"
 
-  external js_expr : string -> 'a = "caml_js_expr"
+    external js_expr : string -> 'a = "caml_js_expr"
 
-  external pure_js_expr : string -> 'a = "caml_pure_js_expr"
+    external pure_js_expr : string -> 'a = "caml_pure_js_expr"
 
-  let global = pure_js_expr "joo_global_object"
+    let global = pure_js_expr "globalThis"
 
-  external callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback = "%identity"
+    external callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
+      = "caml_js_wrap_callback_unsafe"
 
-  external callback_with_arguments :
-    (any_js_array -> 'b) -> ('c, any_js_array -> 'b) meth_callback
-    = "caml_js_wrap_callback_arguments"
+    external callback_with_arguments :
+      (any_js_array -> 'b) -> ('c, any_js_array -> 'b) meth_callback
+      = "caml_js_wrap_callback_arguments"
 
-  external callback_with_arity : int -> ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
-    = "caml_js_wrap_callback_strict"
+    external callback_with_arity : int -> ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
+      = "caml_js_wrap_callback_strict"
 
-  external meth_callback : ('b -> 'a) -> ('b, 'a) meth_callback
-    = "caml_js_wrap_meth_callback_unsafe"
+    external meth_callback : ('b -> 'a) -> ('b, 'a) meth_callback
+      = "caml_js_wrap_meth_callback_unsafe"
 
-  external meth_callback_with_arity : int -> ('b -> 'a) -> ('b, 'a) meth_callback
-    = "caml_js_wrap_meth_callback_strict"
+    external meth_callback_with_arity : int -> ('b -> 'a) -> ('b, 'a) meth_callback
+      = "caml_js_wrap_meth_callback_strict"
 
-  external meth_callback_with_arguments :
-    ('b -> any_js_array -> 'a) -> ('b, any_js_array -> 'a) meth_callback
-    = "caml_js_wrap_meth_callback_arguments"
+    external meth_callback_with_arguments :
+      ('b -> any_js_array -> 'a) -> ('b, any_js_array -> 'a) meth_callback
+      = "caml_js_wrap_meth_callback_arguments"
 
-  (* DEPRECATED *)
-  external variable : string -> 'a = "caml_js_var"
-end
-
-(****)
-
-type 'a opt = 'a
-
-type 'a optdef = 'a
-
-external debugger : unit -> unit = "debugger"
-
-let null : 'a opt = Unsafe.pure_js_expr "null"
-
-external some : 'a -> 'a opt = "%identity"
-
-let undefined : 'a optdef = Unsafe.pure_js_expr "undefined"
-
-external def : 'a -> 'a optdef = "%identity"
-
-module type OPT = sig
-  type 'a t
-
-  val empty : 'a t
-
-  val return : 'a -> 'a t
-
-  val map : 'a t -> ('a -> 'b) -> 'b t
-
-  val bind : 'a t -> ('a -> 'b t) -> 'b t
-
-  val test : 'a t -> bool
-
-  val iter : 'a t -> ('a -> unit) -> unit
-
-  val case : 'a t -> (unit -> 'b) -> ('a -> 'b) -> 'b
-
-  val get : 'a t -> (unit -> 'a) -> 'a
-
-  val option : 'a option -> 'a t
-
-  val to_option : 'a t -> 'a option
-end
-
-module Opt : OPT with type 'a t = 'a opt = struct
-  type 'a t = 'a opt
-
-  let empty = null
-
-  let return = some
-
-  let map x f = if Unsafe.equals x null then null else return (f x)
-
-  let bind x f = if Unsafe.equals x null then null else f x
-
-  let test x = not (Unsafe.equals x null)
-
-  let iter x f = if not (Unsafe.equals x null) then f x
-
-  let case x f g = if Unsafe.equals x null then f () else g x
-
-  let get x f = if Unsafe.equals x null then f () else x
-
-  let option x =
-    match x with
-    | None -> empty
-    | Some x -> return x
-
-  let to_option x = case x (fun () -> None) (fun x -> Some x)
-end
-
-module Optdef : OPT with type 'a t = 'a optdef = struct
-  type 'a t = 'a optdef
-
-  let empty = undefined
-
-  let return = def
-
-  let map x f = if x == undefined then undefined else return (f x)
-
-  let bind x f = if x == undefined then undefined else f x
-
-  let test x = x != undefined
-
-  let iter x f = if x != undefined then f x
-
-  let case x f g = if x == undefined then f () else g x
-
-  let get x f = if x == undefined then f () else x
-
-  let option x =
-    match x with
-    | None -> empty
-    | Some x -> return x
-
-  let to_option x = case x (fun () -> None) (fun x -> Some x)
-end
-
-(****)
-
-let coerce x f g = Opt.get (f x) (fun () -> g x)
-
-let coerce_opt x f g = Opt.get (Opt.bind x f) (fun () -> g x)
-
-(****)
-
-type +'a meth
-
-type +'a gen_prop
-
-type 'a readonly_prop = < get : 'a > gen_prop
-
-type 'a writeonly_prop = < set : 'a -> unit > gen_prop
-
-type 'a prop = < get : 'a ; set : 'a -> unit > gen_prop
-
-type 'a optdef_prop = < get : 'a optdef ; set : 'a -> unit > gen_prop
-
-type +'a constr
-
-(****)
-
-type 'a callback = (unit, 'a) meth_callback
-
-external wrap_callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
-  = "caml_js_wrap_callback"
-
-external wrap_meth_callback : ('a -> 'b) -> ('a, 'b) meth_callback
-  = "caml_js_wrap_meth_callback"
-
-(****)
-
-let _true = Unsafe.pure_js_expr "true"
-
-let _false = Unsafe.pure_js_expr "false"
-
-type match_result_handle
-
-type string_array
-
-class type js_string =
-  object
-    method toString : js_string t meth
-
-    method valueOf : js_string t meth
-
-    method charAt : int -> js_string t meth
-
-    method charCodeAt : int -> float meth
-
-    (* This may return NaN... *)
-    method concat : js_string t -> js_string t meth
-
-    method concat_2 : js_string t -> js_string t -> js_string t meth
-
-    method concat_3 : js_string t -> js_string t -> js_string t -> js_string t meth
-
-    method concat_4 :
-      js_string t -> js_string t -> js_string t -> js_string t -> js_string t meth
-
-    method indexOf : js_string t -> int meth
-
-    method indexOf_from : js_string t -> int -> int meth
-
-    method lastIndexOf : js_string t -> int meth
-
-    method lastIndexOf_from : js_string t -> int -> int meth
-
-    method localeCompare : js_string t -> float meth
-
-    method _match : regExp t -> match_result_handle t opt meth
-
-    method replace : regExp t -> js_string t -> js_string t meth
-
-    method replace_string : js_string t -> js_string t -> js_string t meth
-
-    method search : regExp t -> int meth
-
-    method slice : int -> int -> js_string t meth
-
-    method slice_end : int -> js_string t meth
-
-    method split : js_string t -> string_array t meth
-
-    method split_limited : js_string t -> int -> string_array t meth
-
-    method split_regExp : regExp t -> string_array t meth
-
-    method split_regExpLimited : regExp t -> int -> string_array t meth
-
-    method substring : int -> int -> js_string t meth
-
-    method substring_toEnd : int -> js_string t meth
-
-    method toLowerCase : js_string t meth
-
-    method toLocaleLowerCase : js_string t meth
-
-    method toUpperCase : js_string t meth
-
-    method toLocaleUpperCase : js_string t meth
-
-    method trim : js_string t meth
-
-    method length : int readonly_prop
+    (* DEPRECATED *)
+    external variable : string -> 'a = "caml_js_var"
   end
 
-and regExp =
-  object
-    method exec : js_string t -> match_result_handle t opt meth
+  (****)
 
-    method test : js_string t -> bool t meth
+  type 'a opt = 'a
 
-    method toString : js_string t meth
+  type 'a optdef = 'a
 
-    method source : js_string t readonly_prop
+  external debugger : unit -> unit = "debugger"
 
-    method global : bool t readonly_prop
+  let null : 'a opt = Unsafe.pure_js_expr "null"
 
-    method ignoreCase : bool t readonly_prop
+  external some : 'a -> 'a opt = "%identity"
 
-    method multiline : bool t readonly_prop
+  let undefined : 'a optdef = Unsafe.pure_js_expr "undefined"
 
-    method lastIndex : int prop
+  external def : 'a -> 'a optdef = "%identity"
+
+  module type OPT = sig
+    type 'a t
+
+    val empty : 'a t
+
+    val return : 'a -> 'a t
+
+    val map : 'a t -> ('a -> 'b) -> 'b t
+
+    val bind : 'a t -> ('a -> 'b t) -> 'b t
+
+    val test : 'a t -> bool
+
+    val iter : 'a t -> ('a -> unit) -> unit
+
+    val case : 'a t -> (unit -> 'b) -> ('a -> 'b) -> 'b
+
+    val get : 'a t -> (unit -> 'a) -> 'a
+
+    val option : 'a option -> 'a t
+
+    val to_option : 'a t -> 'a option
   end
 
+  module Opt : OPT with type 'a t = 'a opt = struct
+    type 'a t = 'a opt
+
+    let empty = null
+
+    let return = some
+
+    let map x f = if Unsafe.equals x null then null else return (f x)
+
+    let bind x f = if Unsafe.equals x null then null else f x
+
+    let test x = not (Unsafe.equals x null)
+
+    let iter x f = if not (Unsafe.equals x null) then f x
+
+    let case x f g = if Unsafe.equals x null then f () else g x
+
+    let get x f = if Unsafe.equals x null then f () else x
+
+    let option x =
+      match x with
+      | None -> empty
+      | Some x -> return x
+
+    let to_option x = case x (fun () -> None) (fun x -> Some x)
+  end
+
+  module Optdef : OPT with type 'a t = 'a optdef = struct
+    type 'a t = 'a optdef
+
+    let empty = undefined
+
+    let return = def
+
+    let map x f = if x == undefined then undefined else return (f x)
+
+    let bind x f = if x == undefined then undefined else f x
+
+    let test x = x != undefined
+
+    let iter x f = if x != undefined then f x
+
+    let case x f g = if x == undefined then f () else g x
+
+    let get x f = if x == undefined then f () else x
+
+    let option x =
+      match x with
+      | None -> empty
+      | Some x -> return x
+
+    let to_option x = case x (fun () -> None) (fun x -> Some x)
+  end
+
+  (****)
+
+  let coerce x f g = Opt.get (f x) (fun () -> g x)
+
+  let coerce_opt x f g = Opt.get (Opt.bind x f) (fun () -> g x)
+
+  (****)
+
+  type +'a meth
+
+  type +'a gen_prop
+
+  type 'a readonly_prop = < get : 'a > gen_prop
+
+  type 'a writeonly_prop = < set : 'a -> unit > gen_prop
+
+  type 'a prop = < get : 'a ; set : 'a -> unit > gen_prop
+
+  type 'a optdef_prop = < get : 'a optdef ; set : 'a -> unit > gen_prop
+
+  type +'a constr
+
+  (****)
+
+  type 'a callback = (unit, 'a) meth_callback
+
+  external wrap_callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
+    = "caml_js_wrap_callback"
+
+  external wrap_meth_callback : ('a -> 'b) -> ('a, 'b) meth_callback
+    = "caml_js_wrap_meth_callback"
+
+  (****)
+
+  let _true = Unsafe.pure_js_expr "true"
+
+  let _false = Unsafe.pure_js_expr "false"
+
+  type match_result_handle
+
+  type string_array
+
+  class type js_string =
+    object
+      method toString : js_string t meth
+
+      method valueOf : js_string t meth
+
+      method charAt : int -> js_string t meth
+
+      method charCodeAt : int -> float meth
+
+      (* This may return NaN... *)
+      method concat : js_string t -> js_string t meth
+
+      method concat_2 : js_string t -> js_string t -> js_string t meth
+
+      method concat_3 : js_string t -> js_string t -> js_string t -> js_string t meth
+
+      method concat_4 :
+        js_string t -> js_string t -> js_string t -> js_string t -> js_string t meth
+
+      method indexOf : js_string t -> int meth
+
+      method indexOf_from : js_string t -> int -> int meth
+
+      method lastIndexOf : js_string t -> int meth
+
+      method lastIndexOf_from : js_string t -> int -> int meth
+
+      method localeCompare : js_string t -> float meth
+
+      method _match : regExp t -> match_result_handle t opt meth
+
+      method normalize : js_string t meth
+
+      method normalize_form : normalization t -> js_string t meth
+
+      method replace : regExp t -> js_string t -> js_string t meth
+
+      method replace_string : js_string t -> js_string t -> js_string t meth
+
+      method search : regExp t -> int meth
+
+      method slice : int -> int -> js_string t meth
+
+      method slice_end : int -> js_string t meth
+
+      method split : js_string t -> string_array t meth
+
+      method split_limited : js_string t -> int -> string_array t meth
+
+      method split_regExp : regExp t -> string_array t meth
+
+      method split_regExpLimited : regExp t -> int -> string_array t meth
+
+      method substring : int -> int -> js_string t meth
+
+      method substring_toEnd : int -> js_string t meth
+
+      method toLowerCase : js_string t meth
+
+      method toLocaleLowerCase : js_string t meth
+
+      method toUpperCase : js_string t meth
+
+      method toLocaleUpperCase : js_string t meth
+
+      method trim : js_string t meth
+
+      method length : int readonly_prop
+    end
+
+  and regExp =
+    object
+      method exec : js_string t -> match_result_handle t opt meth
+
+      method test : js_string t -> bool t meth
+
+      method toString : js_string t meth
+
+      method source : js_string t readonly_prop
+
+      method global : bool t readonly_prop
+
+      method ignoreCase : bool t readonly_prop
+
+      method multiline : bool t readonly_prop
+
+      method lastIndex : int prop
+    end
+
+  and normalization = js_string
+
+  (* string is used by ppx_js, it needs to come before any use of the
+     new syntax in this file *)
+  external string : string -> js_string t = "caml_jsstring_of_string"
+
+  external to_string : js_string t -> string = "caml_string_of_jsstring"
+
+  let nfc = string "NFC"
+
+  let nfd = string "NFD"
+
+  let nfkc = string "NFKC"
+
+  let nfkd = string "NFKD"
+end
+
+include Js
+
 class type string_constr =
   object
     method fromCharCode : int -> js_string t meth
@@ -660,13 +686,55 @@ class type error =
     method toString : js_string t meth
   end
 
-exception Error of error t
-
 let error_constr = Unsafe.global##._Error
 
-let _ = Callback.register_exception "jsError" (Error (Unsafe.obj [||]))
+module Js_error = struct
+  type error_t = error t
 
-let raise_js_error : error t -> 'a = Unsafe.js_expr "(function (exn) { throw exn })"
+  include Jsoo_runtime.Error
+
+  external of_error : error_t -> t = "%identity"
+
+  external to_error : t -> error_t = "%identity"
+
+  let name e = to_string (to_error e)##.name
+
+  let message e = to_string (to_error e)##.message
+
+  let stack (e : t) : string option =
+    Opt.to_option (Opt.map (to_error e)##.stack to_string)
+
+  let to_string e = to_string (to_error e)##toString
+end
+
+module Magic = struct
+  module type T = sig
+    exception Error of error t
+  end
+
+  type ('a, 'b) eq = Eq : ('a, 'a) eq
+
+  let (eq : (error t, Js_error.t) eq) = Obj.magic Eq
+
+  let m =
+    match eq with
+    | Eq ->
+        (module struct
+          exception Error = Js_error.Exn
+        end : T)
+
+  module Error = (val m : T)
+end
+
+include Magic.Error
+
+let raise_js_error e = Js_error.raise_ (Js_error.of_error e)
+
+let string_of_error e = Js_error.to_string (Js_error.of_error e)
+
+let exn_with_js_backtrace = Js_error.attach_js_backtrace
+
+external js_error_of_exn : exn -> error t opt = "caml_js_error_of_exception"
 
 class type json =
   object
@@ -699,10 +767,6 @@ external bool : bool -> bool t = "caml_js_from_bool"
 
 external to_bool : bool t -> bool = "caml_js_to_bool"
 
-external string : string -> js_string t = "caml_jsstring_of_string"
-
-external to_string : js_string t -> string = "caml_string_of_jsstring"
-
 external array : 'a array -> 'a js_array t = "caml_js_from_array"
 
 external to_array : 'a js_array t -> 'a array = "caml_js_to_array"
@@ -728,7 +792,7 @@ let parseFloat (s : js_string t) : float =
 
 let _ =
   Printexc.register_printer (function
-      | Error e -> Some (to_string e##toString)
+      | Js_error.Exn e -> Some (Js_error.to_string e)
       | _ -> None)
 
 let _ =
@@ -736,11 +800,15 @@ let _ =
       let e : < .. > t = Obj.magic e in
       if instanceof e array_constructor then None else Some (to_string e##toString))
 
-let string_of_error e = to_string e##toString
-
-external get_export_var : unit -> < .. > t = "caml_js_export_var"
-
-let export_js (field : js_string t) x = Unsafe.set (get_export_var ()) field x
+let export_js (field : js_string t) x =
+  Unsafe.set
+    (Unsafe.pure_js_expr "jsoo_exports")
+    field
+    (if String.equal (Js.to_string (typeof (Obj.magic x))) "function"
+        (* function with arity/length equal to zero are already wrapped *)
+        && Unsafe.get (Obj.magic x) (Js.string "length") > 0
+     then Obj.magic (wrap_callback (Obj.magic x))
+     else x)
 
 let export field x = export_js (string field) x
 
@@ -758,21 +826,3 @@ type float_prop = float prop
 external float : float -> float = "%identity"
 
 external to_float : float -> float = "%identity"
-
-[@@@ocaml.warning "-32-60"]
-
-module For_compatibility_only = struct
-  (* Add primitives for compatibility reasons. Existing users might
-     depend on it (e.g. gen_js_api), we dont want the ocaml compiler
-     to complain about theses missing primitives. *)
-
-  external caml_js_from_string : string -> js_string t = "caml_js_from_string"
-
-  external caml_js_to_byte_string : js_string t -> string = "caml_js_to_byte_string"
-
-  external caml_js_to_string : js_string t -> string = "caml_js_to_string"
-
-  external caml_list_of_js_array : 'a js_array t -> 'a list = "caml_list_of_js_array"
-
-  external caml_list_to_js_array : 'a list -> 'a js_array t = "caml_list_to_js_array"
-end
