@@ -137,7 +137,7 @@ type +'a constr
 (** A value of type [(t1 -> ... -> tn -> t Js.t) Js.constr] is a
       Javascript constructor expecting {i n} arguments of types [t1]
       to [tn] and returning a Javascript object of type [t Js.t].  Use
-      the syntax extension [new%js c (e1, ..., en)] to build an object
+      the syntax extension [new%js c e1 ... en] to build an object
       using constructor [c] and arguments [e1] to [en]. *)
 
 (** {2 Callbacks to OCaml} *)
@@ -188,6 +188,21 @@ type string_array
       (This type is used to resolved the mutual dependency between
        string and array type definitions.) *)
 
+type normalization
+(** Opaque type for Unicode normalization forms. *)
+
+val nfd : normalization t
+(** Canonical Decomposition *)
+
+val nfc : normalization t
+(** Canonical Decomposition, followed by Canonical Composition *)
+
+val nfkd : normalization t
+(** Compatibility Decomposition *)
+
+val nfkc : normalization t
+(** Compatibility Decomposition, followed by Canonical Composition *)
+
 (** Specification of Javascript string objects. *)
 class type js_string =
   object
@@ -221,6 +236,10 @@ class type js_string =
 
     method _match : regExp t -> match_result_handle t opt meth
 
+    method normalize : js_string t meth
+
+    method normalize_form : normalization t -> js_string t meth
+
     method replace : regExp t -> js_string t -> js_string t meth
 
     (* FIX: version of replace taking a function... *)
@@ -287,17 +306,17 @@ val string_constr : string_constr t
 (** The string constructor, as an object. *)
 
 val regExp : (js_string t -> regExp t) constr
-(** Constructor of [RegExp] objects.  The expression [new%js regExp (s)]
+(** Constructor of [RegExp] objects.  The expression [new%js regExp s]
       builds the regular expression specified by string [s]. *)
 
 val regExp_withFlags : (js_string t -> js_string t -> regExp t) constr
 (** Constructor of [RegExp] objects.  The expression
-      [new%js regExp (s, f)] builds the regular expression specified by
+      [new%js regExp s f] builds the regular expression specified by
       string [s] using flags [f]. *)
 
 val regExp_copy : (regExp t -> regExp t) constr
 (** Constructor of [RegExp] objects.  The expression
-      [new%js regExp (r)] builds a copy of regular expression [r]. *)
+      [new%js regExp r] builds a copy of regular expression [r]. *)
 
 (** Specification of Javascript regular arrays.
     Use [Js.array_get] and [Js.array_set] to access and set array elements. *)
@@ -379,11 +398,11 @@ val object_keys : 'a t -> js_string t js_array t
 
 val array_empty : 'a js_array t constr
 (** Constructor of [Array] objects.  The expression
-      [new%js array_empty ()] returns an empty array. *)
+      [new%js array_empty] returns an empty array. *)
 
 val array_length : (int -> 'a js_array t) constr
 (** Constructor of [Array] objects.  The expression
-      [new%js array_length (l)] returns an array of length [l]. *)
+      [new%js array_length l] returns an array of length [l]. *)
 
 val array_get : 'a #js_array t -> int -> 'a optdef
 (** Array access: [array_get a i] returns the element at index [i]
@@ -539,38 +558,38 @@ class type date =
   end
 
 val date_now : date t constr
-(** Constructor of [Date] objects: [new%js date_now ()] returns a
+(** Constructor of [Date] objects: [new%js date_now] returns a
       [Date] object initialized with the current date. *)
 
 val date_fromTimeValue : (float -> date t) constr
-(** Constructor of [Date] objects: [new%js date_fromTimeValue (t)] returns a
+(** Constructor of [Date] objects: [new%js date_fromTimeValue t] returns a
       [Date] object initialized with the time value [t]. *)
 
 val date_month : (int -> int -> date t) constr
-(** Constructor of [Date] objects: [new%js date_fromTimeValue (y, m)]
+(** Constructor of [Date] objects: [new%js date_fromTimeValue y m]
       returns a [Date] object corresponding to year [y] and month [m]. *)
 
 val date_day : (int -> int -> int -> date t) constr
-(** Constructor of [Date] objects: [new%js date_fromTimeValue (y, m, d)]
+(** Constructor of [Date] objects: [new%js date_fromTimeValue y m d]
       returns a [Date] object corresponding to year [y], month [m] and
       day [d]. *)
 
 val date_hour : (int -> int -> int -> int -> date t) constr
-(** Constructor of [Date] objects: [new%js date_fromTimeValue (y, m, d, h)]
+(** Constructor of [Date] objects: [new%js date_fromTimeValue y m d h]
       returns a [Date] object corresponding to year [y] to hour [h]. *)
 
 val date_min : (int -> int -> int -> int -> int -> date t) constr
-(** Constructor of [Date] objects: [new%js date_fromTimeValue (y, m, d, h, m')]
+(** Constructor of [Date] objects: [new%js date_fromTimeValue y m d h m']
       returns a [Date] object corresponding to year [y] to minute [m']. *)
 
 val date_sec : (int -> int -> int -> int -> int -> int -> date t) constr
 (** Constructor of [Date] objects:
-      [new%js date_fromTimeValue (y, m, d, h, m', s)]
+      [new%js date_fromTimeValue y m d h m' s]
       returns a [Date] object corresponding to year [y] to second [s]. *)
 
 val date_ms : (int -> int -> int -> int -> int -> int -> int -> date t) constr
 (** Constructor of [Date] objects:
-      [new%js date_fromTimeValue (y, m, d, h, m', s, ms)]
+      [new%js date_fromTimeValue y m d h m' s ms]
       returns a [Date] object corresponding to year [y]
       to millisecond [ms]. *)
 
@@ -678,19 +697,47 @@ class type error =
 
 val error_constr : (js_string t -> error t) constr
 (** Constructor of [Error] objects:
-      [new%js error_constr (msg)]
+      [new%js error_constr msg]
       returns an [Error] object with the message [msg]. *)
 
-val string_of_error : error t -> string
+module Js_error : sig
+  type error_t = error t
 
-val raise_js_error : error t -> 'a
+  type t
 
-exception Error of error t
-(** The [Error] exception wrap javascript exceptions when caught by OCaml code.
+  val to_string : t -> string
+
+  val name : t -> string
+
+  val message : t -> string
+
+  val stack : t -> string option
+
+  val raise_ : t -> 'a
+
+  val attach_js_backtrace : exn -> force:bool -> exn
+  (** Attach a JavasScript error to an OCaml exception.  if [force = false] and a
+    JavasScript error is already attached, it will do nothing. This function is useful to
+    store and retrieve information about JavaScript stack traces.
+
+    Attaching JavasScript errors will happen automatically when compiling with
+    [--enable with-js-error]. *)
+
+  val of_exn : exn -> t option
+  (** Extract a JavaScript error attached to an OCaml exception, if any.  This is useful to
+      inspect an eventual stack strace, especially when sourcemap is enabled. *)
+
+  exception Exn of t
+  (** The [Error] exception wrap javascript exceptions when caught by OCaml code.
       In case the javascript exception is not an instance of javascript [Error],
       it will be serialized and wrapped into a [Failure] exception.
   *)
 
+  val of_error : error_t -> t
+
+  val to_error : t -> error_t
+end
+
 (** Specification of Javascript JSON object. *)
 class type json =
   object
@@ -865,7 +912,9 @@ module Unsafe : sig
   external new_obj : 'a -> any array -> 'b = "caml_js_new"
   (** Create a Javascript object.  The expression [new_obj c a]
         creates a Javascript object with constructor [c] using the
-        arguments given by the array [a]. *)
+        arguments given by the array [a].
+        Example: [Js.new_obj (Js.Unsafe.variable "ArrayBuffer") [||]]
+  *)
 
   external new_obj_arr : 'a -> any_js_array -> 'b = "caml_ojs_new_arr"
   (** Same Create a Javascript object.  The expression [new_obj_arr c a]
@@ -895,7 +944,8 @@ module Unsafe : sig
   val global : < .. > t
   (** Javascript global object *)
 
-  external callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback = "%identity"
+  external callback : ('a -> 'b) -> ('c, 'a -> 'b) meth_callback
+    = "caml_js_wrap_callback_unsafe"
   (** Wrap an OCaml function so that it can be invoked from
         Javascript. Contrary to [Js.wrap_callback], partial
         application and over-application are not supported: missing
@@ -934,17 +984,48 @@ module Unsafe : sig
   (** {3 Deprecated functions.} *)
 
   external variable : string -> 'a = "caml_js_var"
+    [@@ocaml.deprecated "[since 2.6] use Js.Unsafe.pure_js_expr instead"]
   (** Access a Javascript variable.  [variable "foo"] will
         return the current value of variable [foo]. *)
 end
 
 (** {2 Deprecated functions and types.} *)
 
-external float : float -> float = "%identity"
+val string_of_error : error t -> string
+  [@@ocaml.deprecated "[since 4.0] Use [Js_error.to_string] instead."]
+
+val raise_js_error : error t -> 'a
+  [@@ocaml.deprecated "[since 4.0] Use [Js_error.raise_] instead."]
+
+val exn_with_js_backtrace : exn -> force:bool -> exn
+  [@@ocaml.deprecated "[since 4.0] Use [Js_error.raise_] instead."]
+(** Attach a JavasScript error to an OCaml exception.  if [force = false] and a
+    JavasScript error is already attached, it will do nothing. This function is useful to
+    store and retrieve information about JavaScript stack traces.
+
+    Attaching JavasScript errors will happen automatically when compiling with
+    [--enable with-js-error].
+*)
+
+val js_error_of_exn : exn -> error t opt
+  [@@ocaml.deprecated "[since 4.0] Use [Js_error.of_exn] instead."]
+(** Extract a JavaScript error attached to an OCaml exception, if any.  This is useful to
+    inspect an eventual stack strace, especially when sourcemap is enabled. *)
+
+exception Error of error t [@ocaml.deprecated "[since 4.0] Use [Js_error.Exn] instead."]
+(** The [Error] exception wrap javascript exceptions when caught by OCaml code.
+    In case the javascript exception is not an instance of javascript [Error],
+    it will be serialized and wrapped into a [Failure] exception.
+  *)
+
+external float : float -> float = "%identity" [@@ocaml.deprecated "[since 2.0]."]
+
 (** Conversion of OCaml floats to Javascript numbers. *)
 
-external to_float : float -> float = "%identity"
+external to_float : float -> float = "%identity" [@@ocaml.deprecated "[since 2.0]."]
+
 (** Conversion of Javascript numbers to OCaml floats. *)
 
-type float_prop = float prop
+type float_prop = float prop [@@ocaml.deprecated "[since 2.0]."]
+
 (** Type of float properties. *)
