@@ -172,6 +172,15 @@ let limited_loop event ?(elapsed_time = 0.1) =
 let click ?use_capture ?passive target =
   make_event Dom_html.Event.click ?use_capture ?passive target
 
+let copy ?use_capture ?passive target =
+  make_event Dom_html.Event.copy ?use_capture ?passive target
+
+let cut ?use_capture ?passive target =
+  make_event Dom_html.Event.cut ?use_capture ?passive target
+
+let paste ?use_capture ?passive target =
+  make_event Dom_html.Event.paste ?use_capture ?passive target
+
 let dblclick ?use_capture ?passive target =
   make_event Dom_html.Event.dblclick ?use_capture ?passive target
 
@@ -330,9 +339,12 @@ let mousewheel ?use_capture ?passive target =
   t
 
 (* let _DOMMouseScroll ?use_capture ?passive target =
-  make_event Dom_html.Event._DOMMouseScroll ?use_capture ?passive target
+   make_event Dom_html.Event._DOMMouseScroll ?use_capture ?passive target
 *)
 
+let wheel ?use_capture ?passive target =
+  make_event Dom_html.Event.wheel ?use_capture ?passive target
+
 let touchstart ?use_capture ?passive target =
   make_event Dom_html.Event.touchstart ?use_capture ?passive target
 
@@ -375,9 +387,30 @@ let pointerover ?use_capture ?passive target =
 let pointerup ?use_capture ?passive target =
   make_event Dom_html.Event.pointerup ?use_capture ?passive target
 
+let transitionend ?use_capture ?passive elt =
+  make_event Dom_html.Event.transitionend ?use_capture ?passive elt
+
+let transitionstart ?use_capture ?passive elt =
+  make_event Dom_html.Event.transitionstart ?use_capture ?passive elt
+
+let transitionrun ?use_capture ?passive elt =
+  make_event Dom_html.Event.transitionrun ?use_capture ?passive elt
+
+let transitioncancel ?use_capture ?passive elt =
+  make_event Dom_html.Event.transitioncancel ?use_capture ?passive elt
+
 let clicks ?cancel_handler ?use_capture ?passive t =
   seq_loop click ?cancel_handler ?use_capture ?passive t
 
+let copies ?cancel_handler ?use_capture ?passive t =
+  seq_loop copy ?cancel_handler ?use_capture ?passive t
+
+let cuts ?cancel_handler ?use_capture ?passive t =
+  seq_loop cut ?cancel_handler ?use_capture ?passive t
+
+let pastes ?cancel_handler ?use_capture ?passive t =
+  seq_loop paste ?cancel_handler ?use_capture ?passive t
+
 let dblclicks ?cancel_handler ?use_capture ?passive t =
   seq_loop dblclick ?cancel_handler ?use_capture ?passive t
 
@@ -438,6 +471,9 @@ let drops ?cancel_handler ?use_capture ?passive t =
 let mousewheels ?cancel_handler ?use_capture ?passive t =
   seq_loop mousewheel ?cancel_handler ?use_capture ?passive t
 
+let wheels ?cancel_handler ?use_capture ?passive t =
+  seq_loop wheel ?cancel_handler ?use_capture ?passive t
+
 let touchstarts ?cancel_handler ?use_capture ?passive t =
   seq_loop touchstart ?cancel_handler ?use_capture ?passive t
 
@@ -558,36 +594,24 @@ let pointerovers ?cancel_handler ?use_capture ?passive t =
 let pointerups ?cancel_handler ?use_capture ?passive t =
   seq_loop pointerup ?cancel_handler ?use_capture ?passive t
 
-let transition_evn =
-  lazy
-    (let e = Dom_html.createDiv Dom_html.document in
-     try
-       snd
-         (List.find
-            (fun (propname, _evname) -> Js.Unsafe.get e##.style propname != Js.undefined)
-            [ "WebkitTransition", [ Dom.Event.make "webkitTransitionEnd" ]
-            ; "MozTransition", [ Dom.Event.make "transitionend" ]
-            ; ( "OTransition"
-              , [ Dom.Event.make "oTransitionEnd"; Dom.Event.make "otransitionend" ] )
-            ; "transition", [ Dom.Event.make "transitionend" ]
-            ])
-     with Not_found -> [])
+let transitionends ?cancel_handler ?use_capture ?passive t =
+  seq_loop transitionend ?cancel_handler ?use_capture ?passive t
 
-let transitionend elt =
-  match Lazy.force transition_evn with
-  | [] -> Lwt.return ()
-  | l -> Lwt.pick (List.map (fun ev -> make_event ev elt) l) >>= fun _ -> Lwt.return ()
+let transitionstarts ?cancel_handler ?use_capture ?passive t =
+  seq_loop transitionstart ?cancel_handler ?use_capture ?passive t
 
-let transitionends ?cancel_handler elt f =
-  seq_loop
-    (fun ?use_capture:_ ?passive:_ target -> transitionend target)
-    ?cancel_handler
-    elt
-    (fun _ cancel -> f cancel)
+let transitionruns ?cancel_handler ?use_capture ?passive t =
+  seq_loop transitionrun ?cancel_handler ?use_capture ?passive t
+
+let transitioncancels ?cancel_handler ?use_capture ?passive t =
+  seq_loop transitioncancel ?cancel_handler ?use_capture ?passive t
 
 let request_animation_frame () =
   let t, s = Lwt.wait () in
-  Dom_html._requestAnimationFrame (Js.wrap_callback (fun () -> Lwt.wakeup s ()));
+  let (_ : Dom_html.animation_frame_request_id) =
+    Dom_html.window##requestAnimationFrame
+      (Js.wrap_callback (fun (_ : float) -> Lwt.wakeup s ()))
+  in
   t
 
 let onload () = make_event Dom_html.Event.load Dom_html.window
