@@ -25,6 +25,8 @@ function caml_terminfo_backup () { return 0; }
 function caml_terminfo_standout () { return 0; }
 //Provides: caml_terminfo_resume
 function caml_terminfo_resume () { return 0; }
+//Provides: caml_terminfo_rows
+function caml_terminfo_rows () { return 0; }
 //Provides: caml_invoke_traced_function
 //Requires: caml_invalid_argument
 function caml_invoke_traced_function() {
@@ -38,15 +40,19 @@ function caml_get_current_environment() {
 //////////////////////////////////////////////////////////////////////
 
 //Provides: caml_get_section_table
-//Requires: caml_global_data
-function caml_get_section_table () { return caml_global_data.toc; }
-
+//Requires: caml_global_data, caml_failwith
+function caml_get_section_table () {
+  if(!caml_global_data.toc) {
+    caml_failwith("Program not compiled with --toplevel");
+  }
+  return caml_global_data.toc;
+}
 
 //Provides: caml_reify_bytecode
-//Requires: caml_failwith
-function caml_reify_bytecode (code, _sz) {
-  if(joo_global_object.toplevelCompile)
-    return joo_global_object.toplevelCompile(code);
+//Requires: caml_failwith,caml_callback
+function caml_reify_bytecode (code, debug,_digest) {
+  if(globalThis.toplevelCompile)
+    return [0, 0, caml_callback(globalThis.toplevelCompile, [code,debug])];
   else caml_failwith("Toplevel not initialized (toplevelCompile)")
 }
 
@@ -54,8 +60,8 @@ function caml_reify_bytecode (code, _sz) {
 function caml_static_release_bytecode () { return 0; }
 
 //Provides: caml_static_alloc
-//Requires: caml_create_string
-function caml_static_alloc (len) { return caml_create_string (len); }
+//Requires: caml_create_bytes
+function caml_static_alloc (len) { return caml_create_bytes (len); }
 
 //Provides: caml_static_free
 function caml_static_free () { return 0; }
@@ -66,24 +72,3 @@ function caml_realloc_global (len) {
   if (len + 1 > caml_global_data.length) caml_global_data.length = len + 1;
   return 0;
 }
-
-//Provides: caml_dynlink_open_lib
-function caml_dynlink_open_lib () { return 0; }
-
-//Provides: caml_dynlink_close_lib
-function caml_dynlink_close_lib () { return 0; }
-
-//Provides: caml_dynlink_lookup_symbol
-function caml_dynlink_lookup_symbol (_h, _s) {
-  return 1;
-//  try { eval(s.toString()); return 1; } catch (e) { return 0; };
-}
-
-//Provides: caml_dynlink_add_primitive
-//Requires: caml_global_data
-function caml_dynlink_add_primitive () {
-  return caml_global_data.prim_count++;
-}
-
-//Provides: caml_dynlink_get_current_libs
-function caml_dynlink_get_current_libs () { return [0, 0]; }
